#include "RTClib.h"
#include <Stepper.h>
​
RTC_DS3231 rtc;
​
int skip = 0;
const int stepsPerRevolution = 200; //for the motor itself, total steps w/ reduction ~2038
​
//Stepper clockStepper(stepsPerRevolution, 8, 10, 9, 11);
Stepper clockStepperMin(stepsPerRevolution, 11, 9, 10, 8);
Stepper clockStepperHr(stepsPerRevolution, 7, 5, 6, 4);
​
int timer = 100;           // The higher the number, the slower the timing.
// hour:  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12
int ledPins[] = {31, 33, 35, 37, 39, 41, 43, 45, 47, 49, 46, 53};       // an array of pin numbers to which LEDs are attached
​
int pinCount = 12;           // the number of pins (i.e. the length of the array)
​
byte ledState0A = LOW;     // the current state of LED
byte ledState1A = LOW;     // the current state of LED
byte ledState2A = LOW;     // the current state of LED
byte ledState3A = LOW;     // the current state of LED
​
void setup ()
{
  pinMode(12, INPUT_PULLUP);
  pinMode(13, INPUT_PULLUP);
  pinMode(2, INPUT_PULLUP);
  pinMode(3, INPUT_PULLUP);
  pinMode(14, INPUT_PULLUP);
  ​
  //#ifndef ESP8266
  //    while (!Serial); // for Leonardo/Micro/Zero
  //#endif
  ​
  Serial.begin(9600);

  delay(3000); // wait for console opening

  if ( !rtc.begin() )
  {
    Serial.println("Couldn't find RTC");
    while (1);
  }

  if (rtc.lostPower())
  {
    Serial.println("RTC lost power, lets set the time!");
    // following line sets the RTC to the date & time this sketch was compiled
    rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
    // This line sets the RTC with an explicit date & time, for example to set
    // January 21, 2014 at 3am you would call:
    //rtc.adjust(DateTime(2022, 5, 16, 2, 59, 50));
  }

  //rtc.adjust(DateTime(2022, 5, 16, 16, 59, 50));
  //rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));

  Wire.beginTransmission(0x68); // address DS3231
  Wire.write(0x0E); // select register
  Wire.write(0b00011100); // write register bitmap, bit 7 is /EOSC
  Wire.endTransmission();

  clockStepperMin.setSpeed(100);
  clockStepperHr.setSpeed(100);
}
​
void loop()
{
  DateTime now = rtc.now();

  unsigned char offset = 0;
  if ( digitalRead(14) == LOW )
  {
    Serial.println ("GMT");
  }
  else
  {
    offset = 1;
    Serial.println ("BST");
  }

  int buttonForwardValueHr = digitalRead(12);
  if (buttonForwardValueHr == LOW) {
    clockStepperHr.step(15); //advance ~1:00
  }
  int buttonBackwardValueHr = digitalRead(13);
  if (buttonBackwardValueHr == LOW) {
    clockStepperHr.step(-15); //advance ~1:00
  }
  int buttonForwardValueMin = digitalRead(2);
  if (buttonForwardValueMin == LOW) {
    clockStepperMin.step(15); //advance ~1:00
  }
  int buttonBackwardValueMin = digitalRead(3);
  if (buttonBackwardValueMin == LOW) {
    clockStepperMin.step(-15); //advance ~1:00
  }
  Serial.println(String("DateTime::TIMESTAMP_FULL:\t") + now.timestamp(DateTime::TIMESTAMP_FULL));
  //  int buttonForwardValue = digitalRead(4);
  //Serial.print(now.minute(), DEC); Serial.print("  "); Serial.print(now.second()); Serial.print("  "); Serial.println(buttonForwardValue);
  //delay(10);

  //if (buttonForwardValue == LOW) {
  //    Serial.println("advance stepper 34 steps");
  //    clockStepperMin.step(-34); //advance ~1:00
  //}

  if (now.second() == 0)
  {
    skip += 1;
    if (skip != 29)
    {
      //Serial.print("advance stepper 34 steps"); Serial.print(" skip "); Serial.println(skip);
      clockStepperMin.step(-68); //advance ~1:00
      delay(2000);
    }
    else
    {
      skip = 0;
      //Serial.print("1 step skipped to keep correct time"); Serial.print(" skip "); Serial.println(skip);
      clockStepperMin.step(-72); //advance ~1:00 minus 1 step to keep clock in time
      delay(2000);
    }
  }
  else
  {
    digitalWrite(8, LOW);
    digitalWrite(9, LOW);
    digitalWrite(10, LOW);
    digitalWrite(11, LOW);
  }
  ​
  if (now.second() == 5 && now.minute() == 0 )
  {
    unsigned char hour = now.hour() + offset + 11;
    clockStepperHr.step(342); //advance ~1:00
    delay(2000);

    for (int thisPin = 0; thisPin < pinCount; thisPin++)
    {
      unsigned char index = (thisPin + hour) % 12;
      digitalWrite( ledPins[ index ], HIGH );
      delay(timer);
    }

    for (int thisPin = 1; thisPin < pinCount; thisPin++)
    {
      unsigned char index = (thisPin + hour) % 12;
      digitalWrite( ledPins[ index ], LOW );
      delay(timer);
    }
  }
  else
  {
    digitalWrite(4, LOW);
    digitalWrite(5, LOW);
    digitalWrite(6, LOW);
    digitalWrite(7, LOW);
  }
}
